#include <AT89X051.H>

#ifndef _mo3_SERIAL_H_
#define _mo3_SERIAL_H_ 1

#define BAUD 253 // 19200 Baud
#define REC_BUFF_SIZE 10 // size of serial buffer
#define SEND_BUFF_SIZE 20 // size of serial buffer
#define SEND_POWER_UP 1 // uncomment to send power on meassage

#define INSERT 0 // dont modify
#define TAKE 1   // dont modify

// ///////////////////////////////////
// variables /////////////////////////
// ///////////////////////////////////

unsigned char rec_buffer[REC_BUFF_SIZE];
unsigned char rec_at;

bit rec_ready;

unsigned char send_buffer[SEND_BUFF_SIZE];
unsigned char send_at_wr;
unsigned char send_at_rd;
bit send_done;

bit echo;

// ///////////////////////////////////
// prototypes ////////////////////////
// ///////////////////////////////////

void serial_init();
void serial_send_string(unsigned char* give);
void serial_send_char(unsigned char give);
void serial_send_nl();
void serial_transmit();

// ///////////////////////////////////
// interrupts ////////////////////////
// ///////////////////////////////////

void isr_serial() interrupt SIO_VECTOR
{
  unsigned char in;
  if(RI) // receive
  {
    RI = 0;
	in = SBUF;
	if(!( (in >= 'a' && in <= 'z') || (in >= '0' && in <= '9') || (in == '/')  || (in == 0x0D) ))
		return;
	rec_buffer[rec_at] = in;
	if(in == 0x0D || in == '/' || rec_at >= REC_BUFF_SIZE) // 0x0D == CR
	{
	  rec_buffer[rec_at] = 0;
	  rec_ready = 1;
	  rec_at = 0;
  	  return;
	}
	rec_at++;
	if(echo)
      serial_send_char(in);
  }   
  else  // transmit
  {
    serial_transmit();
    TI = 0;
  }
}

// ///////////////////////////////////
// functions /////////////////////////
// ///////////////////////////////////

/*************************************
functions initialises serial communication
*************************************/

void serial_init()
{
  // Timer 1 for Baud-Rate
  SCON = 0x52;

  TMOD &= 0x0F;
  TMOD |= 0x20;

  TR1 = 1; // Timer 1 start

  PCON = 0x80;

  TL1 = BAUD;
  TH1 = BAUD;

  ES = 1;
  EA = 1;

  rec_at = 0;
  send_done = 1;
  rec_ready = 0;

  if(SEND_POWER_UP)
    serial_send_string("PTZ-Webcam 3000\n");
}


/*************************************
functions prints an string on serial
*************************************/

void serial_send_string(unsigned char* give)
{
  char i = 0;
  while(give[i])
    serial_send_char(give[i++]);
}

/*************************************
functions prints an character
*************************************/

void serial_send_char(unsigned char give)
{
	send_buffer[send_at_wr] = give;
	send_at_wr = (send_at_wr+1)%SEND_BUFF_SIZE;
	if(send_done)
		serial_transmit();
}

/*************************************
functions send char out at uart
*************************************/

void serial_transmit()
{
	if(send_at_rd != send_at_wr)
	{
		send_done = 0;
		SBUF = send_buffer[send_at_rd];    
		send_at_rd = (send_at_rd+1)%SEND_BUFF_SIZE;
	}
	else
		send_done = 1;
}


#endif
